@@ -5,7 +5,7 @@ use warnings;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '0.77';
+$VERSION = '0.78';
 
 #----------------------------------------------------------------------------
 
@@ -101,9 +101,9 @@ sub new {
     }
 
     my %OSNAMES;
-    my @rows = $self->{CPANSTATS}->get_query('array',q{SELECT osname,ostitle FROM osname});
+    my @rows = $self->{CPANSTATS}->get_query('array',q{SELECT osname,ostitle FROM osname ORDER BY id});
     for my $row (@rows) {
-        $OSNAMES{lc $row->[0]} = $row->[1];
+        $OSNAMES{lc $row->[0]} ||= $row->[1];
     }
     $self->osnames( \%OSNAMES );
 
